import json
from openapi.providers.aliyun import Client

from examples.config import config

print(config['aliyun'])
client = Client(**config['aliyun'])
client.add_webhook(config['openapi_webhook'])


if __name__ == '__main__':
    # Send sms
    # result = client.request(
    #     'get', 'dysmsapi', 'SendSms', '2017-05-25',
    #     params={
    #         'PhoneNumbers': '',
    #         'SignName': '',
    #         'TemplateCode': '',
    #         'TemplateParam': json.dumps({'code': '123456'})
    #     }
    # )
    # print(result)

    # Vod
    result = client.request(
        'get', 'vod.cn-shanghai', 'GetVideoPlayAuth', '2017-03-21',
        params={
            'VideoId': '7b07afb2b01b4353a1e7f929b66cc667',
            'AuthInfoTimeout': '1800',
        }
    )
    print(result)



from datetime import datetime

from openapi.providers.crm.tanmarket import Client

from examples.config import config

tan_market_config = config['tanmarket']
filepath = tan_market_config.pop('filepath')
client = Client(**tan_market_config)
client.add_webhook(config['openapi_webhook'])

update_fields = {
    'customerId': 501250113003712,
    'fields': [
        {
            'alias': '描述',
            'values': [
                '8天路飞Python训练营'
            ]
        },
        {
            'alias': '推广人',
            'values': [
                '未知'
            ]
        },
        {
            'alias': '渠道来源',
            'values': ['小鹅通']
        },
        {
            'alias': '下单时间',
            'values': [datetime.now().strftime('%Y-%m-%d %X')]
        },
        {
            'alias': '数分训练营',
            'values': ['S020']
        }
    ]
}

update_result = client.request(
    'post', '/v3/customer/update-by-id',
    json=update_fields
)
print(update_result)



import base64
import hashlib
import socket
import struct
from typing import Optional
from Crypto.Cipher import AES

from openapi.exceptions import DisallowedHost

from openapi.enums import IntegerChoices
from openapi.providers.base import BaseClient, BaseResult, Token
from openapi.utils import xml_to_dict


class Code(IntegerChoices):
    FAIL = -1, '失败'
    SUCCESS = 0, '成功'
    INVALID_WHITE_LIST = 2051, 'ip 未在白名单'


class Result(BaseResult):
    msg: Optional[str] = ''


class Client(BaseClient):
    NAME = '小鹅通'
    API_VERSION = ''
    API_BASE_URL = 'https://api.xiaoe-tech.com'

    def __init__(
        self, app_id, secret,
        client_id=None,
        decrypt_key=None, decrypt_token=None
    ):
        super().__init__()
        self.codes = Code

        self.app_id = app_id
        self.secret = secret
        self.client_id = client_id

        self.decrypt_key = decrypt_key
        self.decrypt_token = decrypt_token

    def request(
        self, method, endpoint, params=None, data=None,
        token_request=False
    ) -> Result:
        if not token_request:
            if method == 'get' and params and 'access_token' not in params:
                params['access_token'] = self.access_token

            if method == 'post' and data and 'access_token' not in data:
                data['access_token'] = self.access_token

        request_url = f'{self.API_BASE_URL}{endpoint}'
        response = self._request(method, request_url, params=params, json=data)
        return Result(**response.json()) if response else Result(code=self.codes.FAIL)

    def check_token(self, access_token):
        result = self.request(
            'post', '/xe.user.batch.get/1.0.0',
            data={'access_token': access_token, 'page': 1, 'page_size': 1}
        )
        return result.code == self.codes.SUCCESS

    def fetch_access_token(self):
        result: Result = self.request(
            'get', '/token', params={
                'app_id': self.app_id,
                'secret_key': self.secret,
                'client_id': self.client_id,
                'grant_type': 'client_credential'
            },
            token_request=True
        )
        if result.code == self.codes.SUCCESS:
            self._token = Token(**result.data)

        if result.code == self.codes.INVALID_WHITE_LIST:
            raise DisallowedHost(result.msg)

    def decrypt(self, encrypt, timestamp, nonce, signature):
        data = [self.decrypt_token, timestamp, nonce, encrypt]
        data.sort()

        sign_aligo = hashlib.sha1()
        sign_aligo.update(''.join(data).encode())
        if signature != sign_aligo.hexdigest():
            return Result(code=self.codes.FAIL, msg='签名不一致')

        key = base64.b64decode(f'{self.decrypt_key}=')
        crypter = AES.new(key, AES.MODE_CBC, key[:16])

        plain_text = crypter.decrypt(base64.b64decode(encrypt))
        pad = ord(plain_text[-1:])
        content = plain_text[16:-pad]
        xml_length = socket.ntohl((struct.unpack('I', content[:4]))[0])
        xml_content = content[4:xml_length + 4]
        return Result(
            code=self.codes.SUCCESS, msg='OK',
            data=xml_to_dict(f'<xml>{xml_content.decode()}</xml>')
        )



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# Date: 2023/2/6
from openapi.providers.lenovo import Client, verify_signature
from examples.config import config

lenovo_pay_config = config['lenovopay']
client = Client(**lenovo_pay_config)

response = {
    'mchNo': '1231313212asdaabb',
    'tradeNo': 1047326995117440,
    'serialsNo': '2023020422001432721427426469',
    'payFee': 1,
    'payTime': '2023-02-04 00:52:18',
    'payStatus': '1',
    'attach': '{"mobile": 1880561683}',
    'appId': client.app_id,
    'mchId': client.mch_id,
    'sign': 'iMT2Z/0iT9w+kcdlypXrlYlvVUyb2dCJJ2xv2N0K3HVqFp1wf26ab70QVSY7WpFrQYumo6uZQk9TZy2Al/6U5iIGtECuacy9fxgws+m2VsgpSRGTx6axFAdiD0ILaFYmNfcKi3vu5thBbOOF6p4NKOhs3gy7/bXOnvDlcS6qZxbgjtugKTVPfPuJXWmDs29COLAQUfhuwaGVq6wMBzEVkFKt73G5kt//7zLwkraipcnlsZP4ih5Knqsqhkvvnk/uD1K6rEPqHe0GIDtFlg3j0n+hPvNudVCxU3MPEOiM2puUrm4krZwfBJhiIY/JWZGxUUUWRk0vjD+9g1SrpI9bCg==',
    'signType': 'RSA2',
    'version': '1.0',
    'charSet': 'UTF-8',
    'nonce': 'cjx48mj4vr'
}

print(verify_signature(response, client.public_key))



from openapi.providers.doudian import Client

from examples.config import config

client = Client(**config['doudian'])
client.add_webhook(config['openapi_webhook'])


if __name__ == '__main__':
    result = client.request(
        'post', '/order/orderDetail',
        data={'shop_order_id': '4984023467827309610'}
    )
    print(result)



from typing import Optional
from urllib.parse import quote

from openapi.providers.base import BaseClient, BaseResult, Token
from openapi.exceptions import DisallowedHost, OpenAPIException
from openapi.enums import IntegerChoices


class Code(IntegerChoices):
    FAIL = -1, '失败'
    SUCCESS = 0, '成功'
    INVALID_WHITE_LIST = 40164, 'ip 未在白名单'


class Result(BaseResult):
    errcode: int = Code.SUCCESS
    errmsg: Optional[str]
    msgid: Optional[int]


class Client(BaseClient):
    NAME = '微信服务号'
    API_BASE_URL = 'https://api.weixin.qq.com/cgi-bin'
    API_VERSION = ''

    def __init__(self, app_id, secret):
        super().__init__()
        self.app_id = app_id
        self.secret = secret
        self.codes = Code

    def request(
        self, method, endpoint, params=None, data=None,
        token_request=False, is_oauth=True,
    ):
        if not token_request:
            if params is None:
                params = {}

            if 'access_token' not in params and not is_oauth:
                params['access_token'] = self.access_token

        # Support wechat oauth login
        api_base_url = self.API_BASE_URL
        if not is_oauth:
            api_base_url = self.API_BASE_URL.replace('/cgi-bin', '')

        request_url = f'{api_base_url}{endpoint}'
        response = self._request(
            method, request_url,
            params=params, json=data
        )
        if response is None:
            return Result(code=self.codes.FAIL)

        result = response.json()
        if 'errcode' in result:
            return Result(**result)
        else:
            return Result(data=result)

    def check_token(self, access_token):
        result = self.request(
            'get', '/get_api_domain_ip',
            params={'access_token': access_token}
        )
        return result.code == self.codes.SUCCESS

    def fetch_access_token(self):
        result = self.request(
            'get', '/token', params={
                'grant_type': 'client_credential',
                'appid': self.app_id,
                'secret': self.secret
            },
            token_request=True
        )
        if result.errcode == self.codes.SUCCESS:
            self._token = Token(**result.data)
        else:
            if result.errcode == self.codes.INVALID_WHITE_LIST:
                raise DisallowedHost(result.errmsg)
            else:
                raise OpenAPIException(result.code, result.errmsg)

    def get_authorize_url(
        self, scope='snsapi_base', state='',
        redirect_uri=''
    ):
        redirect_uri = quote(redirect_uri, safe='')
        url_list = [
            'https://open.weixin.qq.com',
            '/connect/oauth2/authorize?appid=',
            self.app_id,
            '&redirect_uri=',
            redirect_uri,
            '&response_type=code&scope=',
            scope
        ]
        if state:
            url_list.extend(['&state=', state])
        url_list.append('#wechat_redirect')
        return ''.join(url_list)

    def get_qrcode_url(self, state='', redirect_uri=''):
        redirect_uri = quote(redirect_uri, safe='')
        url_list = [
            'https://open.weixin.qq.com',
            '/connect/qrconnect?appid=',
            self.app_id,
            '&redirect_uri=',
            redirect_uri,
            '&response_type=code&scope=',
            'snsapi_login'  # scope
        ]
        if state:
            url_list.extend(['&state=', state])
        url_list.append('#wechat_redirect')
        return ''.join(url_list)



from openapi.providers.wechat.open import Client

from examples.config import config

print(config)
client = Client(**config['wechat'])
client.add_webhook(config['openapi_webhook'])

# 网络检测
result = client.request('post', '/callback/check', data={})
print(result)
print(result.errcode == client.codes.SUCCESS_CODE)

# 发送模板消息
result = client.request(
    'post', '/message/template/send',
    data={
        'touser': 'ofwIAuOwpNTVRDBK82KRuH7xhpXo',
        'template_id': 'juLkJRq6c2aijf0W9AZcf2pbfVbAl5qvsd8vD9D4HLI',
        'data': {
            'first': {'value': f'您的学生 谷爱凌 同学，已开通新模块：'},
            'keyword1': {'value': 'Python开发'},
            'keyword2': {'value': '2022-09-03 15:34'},
            'remark': {'value': '开通模块：第一模块'}
        },
        'url': None,
        'miniprogram': None
    }
)
print(result)
print(result.errcode == client.codes.SUCCESS_CODE)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# Date: 2022/11/21
from openapi.providers.feishu.open import Client

from examples.config import config

feishu_open_config = config['feishu']['open']

client = Client(feishu_open_config['app_id'], feishu_open_config['secret'])

# 编辑表格
result = client.request(
    'post', f'/sheets/v2/spreadsheets/{feishu_open_config["table_id"]}/values_prepend',
    json={
        'valueRange': {
            'range': '45BW2x!A2:T2',
            'values': [
                [
                    'string', 10, 'http://www.xx.com', '1231313123', '123sflkadj'
                ]
            ]
        }
    }
)
print(result)



#! /usr/bin/env python
# -*- coding: utf-8 -*-
# Date: 2023/2/3
from Crypto.PublicKey import RSA

from examples.config import config
from openapi.providers.lenovo import calculate_signature, format_params

lenovo_pay_config = config['lenovopay']

data = {"mchNo": "1231313212asd", "payAmount": 100, "goodsName": "table", "goodsDec": "", "goodsCode": "1", "productCode": "COMMON_CASHIER", "attach": "{'x': 1}", "payNotify": "https://api.luffycity.com/pay", "mchId": "x", "appId": "x", "nonce": "40c6ad3ac938a7032d6ec315218dd08d", "timestamp": "1675410455", "version": "1.0", "signType": "RSA2"}
real_sign = 'aROFT8oFCKtKj8W7RSs27J8RJSkydK1BFmC9sInb4OBkOdBMram9U12PSzmtxQXNqg/P4qRjPw/E67uIQY0/pCT7hT/W9pMqknzu2rqpRe5X2viUL3Ar9882XTH6I6IAXbyhLZj4gvnXGcHO1g6rhP8pR6s9FCjek2C888I5HFLtCLuprQ7/RNdbkUs74VgLUN2G8vQzsar5iEENXdYFUUxJPxs8VBRYQq8TttTo+y7FXKI6tUvUK0gIQ3uhIG0NYnYl1bfYnUCz3XkaH3jIGarxC8a9k2so5PEJeRFk34aQuXTHJHjUqaMNSouLQ7sNJBRjohkKV9YVl1wocDd5Sw=='

with open(lenovo_pay_config['private_key_path']) as f:
    pri_key = RSA.importKey(f.read())
    content = format_params(data)
    sign = calculate_signature(
        content.encode(), pri_key
    )
    print(sign == real_sign)



from openapi.providers.wechat.open import Client

from examples.config import config

wechat_config = config['wechat']
mobile_client = Client(app_id=wechat_config['app_id'], secret=wechat_config['secret'])
mobile_client.add_webhook(config['openapi_webhook'])

wechat_open_config = config['wechat_open']
pc_client = Client(app_id=wechat_open_config['app_id'], secret=wechat_open_config['secret'])
pc_client.add_webhook(config['openapi_webhook'])


qrcode_url = mobile_client.get_qrcode_url(
    state='wechat',
    redirect_uri=wechat_config['oauth_redirect_uri']
)
print(qrcode_url)

qrcode_url = pc_client.get_qrcode_url(
    state='wechat',
    redirect_uri=wechat_open_config['oauth_redirect_uri']
)
print(qrcode_url)


authorize_url = mobile_client.get_authorize_url(
    scope='snsapi_userinfo', state='wechat',
    redirect_uri=wechat_config['oauth_redirect_uri']
)
print(authorize_url)



import typing

from openapi.providers.base import BaseClient, BaseResult
from openapi.enums import TextChoices


class Code(TextChoices):
    FAIL = 'error', '失败'
    SUCCESS = 'success', '成功'


class Result(BaseResult):
    status: Code
    send_id: typing.Optional[str]
    fee: typing.Optional[int]
    sms_credits: typing.Optional[int]
    msg: typing.Optional[str]


class Client(BaseClient):
    NAME = '赛邮-云通信'
    API_BASE_URL = 'https://api.mysubmail.com'
    API_VERSION = ''

    def __init__(self, app_id, app_key):
        super(Client, self).__init__()
        self.app_id = app_id
        self.app_key = app_key

        self.codes = Code

    def fetch_access_token(self):
        pass

    def request(
        self, method, endpoint,
        params: typing.Dict = None,
        data: typing.Dict = None,
        json: typing.Dict = None,
    ):
        request_url = f'{self.API_BASE_URL}{endpoint}'
        json.update(**{
            'appid': self.app_id, 'signature': self.app_key
        })
        response = self._request(method, request_url, params, data, json)
        return Result(**(response.json() if response else {'status': self.codes.FAIL}))
